/////////////////////////////////////////////////////////////////////////////
// ActiveXCtl.cpp : Implementation of DLL Exports.
//
// DLL Entry point implementation
//
// All rights reserved.
//
// Written by Naveen Kohli (naveen@a3ds.com)
// Version 1.0
//
// Distribute freely, except: don't remove my name from the source or
// documentation (don't take credit for my work), mark your changes (don't
// get me blamed for your possible bugs), don't alter or remove this
// notice.
// No warrantee of any kind, express or implied, is included with this
// software; use at your own risk, responsibility for damages (if any) to
// anyone resulting from the use of this software rests entirely with the
// user.
//
// Send bug reports, bug fixes, enhancements, requests, flames, etc. to
//    naveen@a3ds.com
/////////////////////////////////////////////////////////////////////////////

// Note: Proxy/Stub Information
//      To build a separate proxy/stub DLL, 
//      run nmake -f ActiveXCtlps.mk in the project directory.

#include "stdafx.h"
#include "resource.h"
#include <initguid.h>
#include <comcat.h>
#include "ActiveXCtl.h"

#include "ActiveXCtl_i.c"
#include "ShapeCtl.h"
#include "ShapeProp.h"
#include "AppearanceProp.h"


CComModule _Module;

BEGIN_OBJECT_MAP(ObjectMap)
OBJECT_ENTRY(CLSID_ShapeCtl, CShapeCtl)
OBJECT_ENTRY(CLSID_ShapeProp, CShapeProp)
OBJECT_ENTRY(CLSID_AppearanceProp, CAppearanceProp)
END_OBJECT_MAP()

// Define the functions for category creation.

HRESULT CreateComponentCategory (CATID catid, WCHAR *catDescription);
HRESULT RegisterCLSIDInCategory (REFCLSID clsId, CATID catId);
HRESULT UnRegisterCLSIDInCategory (REFCLSID clsId, CATID catId);

/////////////////////////////////////////////////////////////////////////////
// DLL Entry Point

extern "C"
BOOL WINAPI DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID /*lpReserved*/)
{
    if (dwReason == DLL_PROCESS_ATTACH)
    {
        _Module.Init(ObjectMap, hInstance, &LIBID_ACTIVEXCTLLib);
        DisableThreadLibraryCalls(hInstance);
    }
    else if (dwReason == DLL_PROCESS_DETACH)
        _Module.Term();
    return TRUE;    // ok
}

/////////////////////////////////////////////////////////////////////////////
// Used to determine whether the DLL can be unloaded by OLE

STDAPI DllCanUnloadNow(void)
{
    return (_Module.GetLockCount()==0) ? S_OK : S_FALSE;
}

/////////////////////////////////////////////////////////////////////////////
// Returns a class factory to create an object of the requested type

STDAPI DllGetClassObject(REFCLSID rclsid, REFIID riid, LPVOID* ppv)
{
    return _Module.GetClassObject(rclsid, riid, ppv);
}

/////////////////////////////////////////////////////////////////////////////
// DllRegisterServer - Adds entries to the system registry

STDAPI DllRegisterServer(void)
{
    HRESULT hRes = S_OK;

	// registers object, typelib and all interfaces in typelib
    hRes = _Module.RegisterServer(TRUE);

	if (FAILED (hRes)) {
		return (hRes);
	}

	// Make the control as safe for initializing.

	hRes = CreateComponentCategory (CATID_SafeForInitializing,
		L"Controls safely initializable from persistent data!");
	if (FAILED (hRes)) {
		return (hRes);
	}

	hRes = RegisterCLSIDInCategory (CLSID_ShapeCtl, CATID_SafeForInitializing);
	if (FAILED (hRes)) {
		return (hRes);
	}

	// Now make safe for scription.

	hRes = CreateComponentCategory (CATID_SafeForScripting, L"Controls safely scriptable!");
	if (FAILED (hRes)) {
		return (hRes);
	}

	hRes = RegisterCLSIDInCategory (CLSID_ShapeCtl, CATID_SafeForScripting);
	if (FAILED (hRes)) {
		return (hRes);
	}

	return (NOERROR);
}

/////////////////////////////////////////////////////////////////////////////
// DllUnregisterServer - Removes entries from the system registry

STDAPI DllUnregisterServer(void)
{
	HRESULT hRes;

	hRes = _Module.UnregisterServer(TRUE);
	if (FAILED (hRes)) {
		return (hRes);
	}
    
	// Unregister the component categories which were registered for safe initialization &
	// safe scripting.

	hRes = UnRegisterCLSIDInCategory (CLSID_ShapeCtl, CATID_SafeForInitializing);
	if (FAILED (hRes)) {
		return (hRes);
	}

	hRes = UnRegisterCLSIDInCategory (CLSID_ShapeCtl, CATID_SafeForScripting);
	if (FAILED (hRes)) {
		return (hRes);
	}

	return NOERROR;
}

/////////////////////////////////////////////////////////////////////////////
// Component Category creation

HRESULT CreateComponentCategory (CATID catid, WCHAR *catDescription)
{
	ICatRegister *pCatReg = NULL;
	HRESULT hRes = S_OK;

	// Create instance of ICatRegister interface

	hRes = CoCreateInstance (CLSID_StdComponentCategoriesMgr, NULL, CLSCTX_INPROC_SERVER,
		IID_ICatRegister, (void **)&pCatReg);

	if (FAILED (hRes)) {
		return hRes;
	}

	// Make sure that HKCR\Component Categories\{..catid..} key is regstered
	
	// Definition CATEGORYINFO structure as defined in comcat.h files.
	//	typedef struct  tagCATEGORYINFO
	//	{
	//	CATID catid;
	//	LCID lcid;
	//	OLECHAR szDescription[ 128 ];
	//	}

	CATEGORYINFO catInfo;
	catInfo.catid = catid;
	catInfo.lcid = 0x0409;		// English
	
	// Check the size of decsription. It shouldn't have more than 127 characters.

	int len = wcslen (catDescription);
	if (len > 127) {
		len = 127;
	}
	
	wcsncpy (catInfo.szDescription, catDescription, len);
		
	// Add the null termination to string.

	catInfo.szDescription[len] = '\0';

	// Register the category.

	hRes = pCatReg->RegisterCategories (1, &catInfo);

	// Release the interface.

	pCatReg->Release ();

	return (hRes);
}

/////////////////////////////////////////////////////////////////////////////////////////
// Register control's clsid in component categories

HRESULT RegisterCLSIDInCategory (REFCLSID clsId, CATID catId)
{
	HRESULT hRes = S_OK;

	ICatRegister *pCatReg = NULL;

	// Create instance of ICatRegister interface.

	hRes = CoCreateInstance (CLSID_StdComponentCategoriesMgr, NULL, CLSCTX_INPROC_SERVER,
		IID_ICatRegister, (void **)&pCatReg);
	if (FAILED (hRes)) {
		return (hRes);
	}

	CATID regCatId[1];
	regCatId[0] = catId;
	
	// Register the class as implementing one or more component categories

	hRes = pCatReg->RegisterClassImplCategories (clsId, 1, regCatId);

	// release ICatRegister interface.

	if (pCatReg != NULL) {
		pCatReg->Release ();
	}

	return (hRes);
}

/////////////////////////////////////////////////////////////////////////////////////////
// Component category unregistration

HRESULT UnRegisterCLSIDInCategory (REFCLSID clsId, CATID catId)
{
	HRESULT hRes = S_OK;
	ICatRegister *pCatReg = NULL;

	// Create instance of the ICatRegister interface.

	hRes = CoCreateInstance (CLSID_StdComponentCategoriesMgr, NULL, CLSCTX_INPROC_SERVER,
		IID_ICatRegister, (void **)&pCatReg);
	if (FAILED (hRes)) {
		return (hRes);
	}

	CATID regCatId[1];
	regCatId[0] = catId;

	// Remove one or more implemented category identifiers from a class

	hRes = pCatReg->UnRegisterClassImplCategories (clsId, 1, regCatId);

	// Release ICatRegister interface.

	if (pCatReg != NULL) {
		pCatReg->Release ();
	}

	return (hRes);
}