/////////////////////////////////////////////////////////////////////////////
// AppearanceProp.cpp : Implementation of CAppearanceProp
//
// Property page for control's properties
//
// All rights reserved.
//
// Written by Naveen Kohli (naveen@a3ds.com)
// Version 1.0
//
// Distribute freely, except: don't remove my name from the source or
// documentation (don't take credit for my work), mark your changes (don't
// get me blamed for your possible bugs), don't alter or remove this
// notice.
// No warrantee of any kind, express or implied, is included with this
// software; use at your own risk, responsibility for damages (if any) to
// anyone resulting from the use of this software rests entirely with the
// user.
//
// Send bug reports, bug fixes, enhancements, requests, flames, etc. to
//    naveen@a3ds.com
/////////////////////////////////////////////////////////////////////////////
 
// 
#include "stdafx.h"
#include "ActiveXCtl.h"
#include "AppearanceProp.h"

/////////////////////////////////////////////////////////////////////////////
// CAppearanceProp

//
//	Function:
//		Apply (void)
//
//	Parameters;
//
//	Return Values:
//		S_OK	Control values applied successfully
//		E_FAIL	Error in applying the control values
//
//	Comments:
//		Function updates the values of control from the values in the property page.
//

STDMETHODIMP
CAppearanceProp::Apply (void)
{
	USES_CONVERSION;
	HRESULT hResult;
	

	// NOTE: There may be different applications referencing to this control. Therefor we have to
	//		 process the loop and get IUnkown pointer for each site and do the processing.

	for (UINT i = 0; i < m_nObjects; i++)
	{
		CComQIPtr<IShapeCtl, &IID_IShapeCtl> pShapeCtl (m_ppUnk[i]);

		// Set appearance

		short appearance;

		appearance = (short)SendDlgItemMessage (IDC_APPEARANCE_COMBO, CB_GETCURSEL, 0, 0);
		if (FAILED (pShapeCtl->put_Appearance (appearance))) {
			HandleError ();
			return (E_FAIL);
		}

		// Set fill style

		long style;

		style = SendDlgItemMessage (IDC_FILLSTYLE_COMBO, CB_GETCURSEL, 0, 0);
		if (FAILED (pShapeCtl->put_FillStyle (style))) {
			HandleError ();
			return (E_FAIL);
		}

		// Set the fill pattern if that kind of fill style has been chosen by user.

		short pattern;
		FILLPATTERN fillPattern;

		if (style == 2) {
			pattern = (short)SendDlgItemMessage (IDC_PATTERN_COMBO, CB_GETCURSEL, 0, 0);
			switch (pattern) {
				case 0:
					fillPattern = fillBDiagonal;
					break;
				case 1:
					fillPattern = fillCrossHatch;
					break;
				case 2:
					fillPattern = fillDiagXHatch;
					break;
				case 3:
					fillPattern = fillFDiagonal;
					break;
				case 4:
					fillPattern = fillHorizontal;
					break;
				case 5:
					fillPattern = fillVertical;
					break;
			}

			if (FAILED (pShapeCtl->put_FillPattern (fillPattern))) {
				HandleError ();
				return (E_FAIL);
			}
		}

		// Set the drwa width

		long drawWidth;

		drawWidth = GetDlgItemInt (IDC_BORDERWIDTH_EDIT, NULL, NULL);
		if (FAILED (pShapeCtl->put_DrawWidth (drawWidth))) {
			HandleError ();
			return (E_FAIL);
		}

		// Set the draw style.

		if (FAILED (pShapeCtl->put_DrawStyle (m_nDrawStyle))) {
			HandleError ();
			return (E_FAIL);
		}

		VARIANT_BOOL varBool;

		hResult = ::SendMessage (m_hEnableCheck, BM_GETCHECK, 0, 0);
		varBool = (hResult == BST_CHECKED) ? -1 : 0;
		if (FAILED (pShapeCtl->put_Enabled (varBool))) {
			HandleError ();
			return (E_FAIL);
		}

		hResult = ::SendMessage (m_hTabStopCheck, BM_GETCHECK, 0, 0);
		varBool = (hResult == BST_CHECKED) ? -1 : 0;
		if (FAILED (pShapeCtl->put_TabStop (varBool))) {
			HandleError ();
			return (E_FAIL);
		}

		hResult = ::SendMessage (m_hBorderVisibleCheck, BM_GETCHECK, 0, 0);
		varBool = (hResult == BST_CHECKED) ? -1 : 0;
		if (FAILED (pShapeCtl->put_BorderVisible (varBool))) {
			HandleError ();
			return (E_FAIL);
		}

	}

	m_bDirty = FALSE;
	return S_OK;
}

//
//	Function:
//		OnInitDialog(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
//
//	Parameters;
//
//	Return Values:
//		S_OK	Successful property page intialization
//		E_FAIL	Error in property page intialization
//
//	Comments:
//		Function handles the property page intialization.
//

LRESULT
CAppearanceProp::OnInitDialog(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	USES_CONVERSION;

	// Lets set some ranges for controls.

	if ((m_hBorderWidthEdit = GetDlgItem (IDC_BORDERWIDTH_EDIT)) != NULL) {
		::SendMessage (m_hBorderWidthEdit, UDM_SETRANGE, 0, MAKELONG (10, 0));
	}

	if ((m_hBorderWidthSpin = GetDlgItem (IDC_BORDERWIDTH_SPIN)) != NULL) {
		::SendMessage (m_hBorderWidthSpin, UDM_SETRANGE, 0, MAKELONG (10, 0));
	}

	m_hAppearanceCombo = GetDlgItem (IDC_APPEARANCE_COMBO);
	m_hFillStyleCombo = GetDlgItem (IDC_FILLSTYLE_COMBO);
	m_hFillPatternCombo = GetDlgItem (IDC_PATTERN_COMBO);
	m_hDrawStyleCombo = GetDlgItem (IDC_DRAWSTYLE_COMBO);
	m_hEnableCheck = GetDlgItem (IDC_ENABLE_CHECK);
	m_hBorderVisibleCheck = GetDlgItem (IDC_BORDERVISIBLE_CHECK);
	m_hTabStopCheck = GetDlgItem (IDC_TABSTOP_CHECK);

	// Initialize all the controls.

	if (m_nObjects > 0) {
		CComQIPtr<IShapeCtl, &IID_IShapeCtl> pShapeCtl (m_ppUnk[0]);
		long style;
		long drawStyle;
		FILLPATTERN pattern;
		short appearance;
		BOOL enableWidthEdit = FALSE;

		// Set the draw (line) width.

		long width;

		if (SUCCEEDED (pShapeCtl->get_DrawWidth (&width))) {
			SetDlgItemInt (IDC_BORDERWIDTH_EDIT, width, FALSE);
			::SendMessage (m_hBorderWidthSpin, UDM_SETPOS, 0, MAKELONG (width, 0));
		}

		// Set the appearance combo box items.

		if (m_hAppearanceCombo != NULL) {
			SendDlgItemMessage (IDC_APPEARANCE_COMBO, CB_ADDSTRING, 0, (long) "0 - Flat");
			SendDlgItemMessage (IDC_APPEARANCE_COMBO, CB_ADDSTRING, 0, (long) "1 - 3D");

			if (SUCCEEDED (pShapeCtl->get_Appearance (&appearance))) {
				::SendMessage (m_hAppearanceCombo, CB_SETCURSEL, appearance, 0);
			}
		}

		// Set the fill style combo box.

		if (m_hFillStyleCombo != NULL) {
			SendDlgItemMessage (IDC_FILLSTYLE_COMBO, CB_ADDSTRING, 0, (long) "0 - Solid");
			SendDlgItemMessage (IDC_FILLSTYLE_COMBO, CB_ADDSTRING, 0, (long) "1 - Transparent");
			SendDlgItemMessage (IDC_FILLSTYLE_COMBO, CB_ADDSTRING, 0, (long) "2 - X-hatched");

			if (SUCCEEDED (pShapeCtl->get_FillStyle (&style))) {
				::SendMessage (m_hFillStyleCombo, CB_SETCURSEL, style, 0);
			}
		}

		// Set the fill pattern combo box.

		if (m_hFillPatternCombo != NULL) {
			SendDlgItemMessage (IDC_PATTERN_COMBO, CB_ADDSTRING, 0, (long) "0 - B_Diagonal");
			SendDlgItemMessage (IDC_PATTERN_COMBO, CB_ADDSTRING, 0, (long) "1 - Cross");
			SendDlgItemMessage (IDC_PATTERN_COMBO, CB_ADDSTRING, 0, (long) "2 - X_Diagonal");
			SendDlgItemMessage (IDC_PATTERN_COMBO, CB_ADDSTRING, 0, (long) "3 - F_Diagonal");
			SendDlgItemMessage (IDC_PATTERN_COMBO, CB_ADDSTRING, 0, (long) "4 - Horizontal");
			SendDlgItemMessage (IDC_PATTERN_COMBO, CB_ADDSTRING, 0, (long) "5 - Vertical");

			if (SUCCEEDED (pShapeCtl->get_FillPattern (&pattern))) {
				::SendMessage (m_hFillPatternCombo, CB_SETCURSEL, (long) pattern, 0);

				::EnableWindow (m_hFillPatternCombo, (style == 2) ? TRUE : FALSE);
			}
		}

		// Set the line styles combo box.

		if (m_hDrawStyleCombo != NULL) {
			SendDlgItemMessage (IDC_DRAWSTYLE_COMBO, CB_ADDSTRING, 0, (long) "0 - Solid");
			SendDlgItemMessage (IDC_DRAWSTYLE_COMBO, CB_ADDSTRING, 0, (long) "1 - Dash");
			SendDlgItemMessage (IDC_DRAWSTYLE_COMBO, CB_ADDSTRING, 0, (long) "2 - Dot");
			SendDlgItemMessage (IDC_DRAWSTYLE_COMBO, CB_ADDSTRING, 0, (long) "3 - Dash_Dot");
			SendDlgItemMessage (IDC_DRAWSTYLE_COMBO, CB_ADDSTRING, 0, (long) "4 - Dash_Dot_Dot");
			SendDlgItemMessage (IDC_DRAWSTYLE_COMBO, CB_ADDSTRING, 0, (long) "5 - Invisible");
			SendDlgItemMessage (IDC_DRAWSTYLE_COMBO, CB_ADDSTRING, 0, (long) "6 - Inside_Frame");

			if (SUCCEEDED (pShapeCtl->get_DrawStyle (&m_nDrawStyle))) {
				// Although the style constants have been defined in wingdi.h in the order we have
				// added in the combo box data. But to be on the safe side, we are translating
				// these to ints. In case MicroSoft decided to rearrange or insert new styles in
				// future, we don't get the hit by change of #defines.

				switch (m_nDrawStyle) {
					case PS_SOLID:	
						drawStyle = 0;
						break;
					case PS_DASH:	
						drawStyle = 1;
						break;
					case PS_DOT:	
						drawStyle = 2;
						break;
					case PS_DASHDOT:	
						drawStyle = 3;
						break;
					case PS_DASHDOTDOT:	
						drawStyle = 4;
						break;
					case PS_NULL:	
						drawStyle = 5;
						break;
					case PS_INSIDEFRAME:	
						drawStyle = 6;
						break;
				}

				::SendMessage (m_hDrawStyleCombo, CB_SETCURSEL, drawStyle, 0);
			}
		}

		// Enable the width edit control only if the line style is one of Invisible, Solid &
		// Inside frame.

		enableWidthEdit = (drawStyle == PS_SOLID || drawStyle == PS_NULL ||
			drawStyle == PS_INSIDEFRAME) ? TRUE : FALSE;

		::EnableWindow (m_hBorderWidthEdit, enableWidthEdit);
		::EnableWindow (m_hBorderWidthSpin, enableWidthEdit);

		// Set the check boxes.

		if (m_hEnableCheck != NULL) {
			VARIANT_BOOL varBool;
			if (SUCCEEDED (pShapeCtl->get_Enabled (&varBool))) {
				::SendMessage (m_hEnableCheck, BM_SETCHECK, (varBool == -1) ? BST_CHECKED : 
				BST_UNCHECKED, 0);
			}
		}

		if (m_hTabStopCheck != NULL) {
			VARIANT_BOOL varBool;
			if (SUCCEEDED (pShapeCtl->get_TabStop (&varBool))) {
				::SendMessage (m_hTabStopCheck, BM_SETCHECK, (varBool == -1) ? BST_CHECKED : 
				BST_UNCHECKED, 0);
			}
		}

		if (m_hBorderVisibleCheck != NULL) {
			VARIANT_BOOL varBool;
			if (SUCCEEDED (pShapeCtl->get_BorderVisible (&varBool))) {
				::SendMessage (m_hBorderVisibleCheck, BM_SETCHECK, (varBool == -1) ? BST_CHECKED : 
				BST_UNCHECKED, 0);
			}
		}
	}

	return 0;
}

//
//	Function:
//		OnSelchangeAppearance_combo(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
//
//	Parameters;
//
//	Return Values:
//
//	Comments:
//		Function handles the message for change in selection in the appearance type combo box.
//

LRESULT
CAppearanceProp::OnSelchangeAppearance_combo(WORD wNotifyCode, WORD wID, HWND hWndCtl,
											 BOOL& bHandled)
{
	SetDirty (TRUE);
	m_pPageSite->OnStatusChange (PROPPAGESTATUS_DIRTY | PROPPAGESTATUS_VALIDATE);

	return 0;
}

//
//	Function:
//		OnClickedBordervisible_check(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
//
//	Parameters;
//
//	Return Values:
//
//	Comments:
//		Function handles message for change in check box state of border visiblity.
//

LRESULT
CAppearanceProp::OnClickedBordervisible_check(WORD wNotifyCode, WORD wID, HWND hWndCtl,
											  BOOL& bHandled)
{
	SetDirty (TRUE);
	m_pPageSite->OnStatusChange (PROPPAGESTATUS_DIRTY | PROPPAGESTATUS_VALIDATE);

	return 0;
}

//
//	Function:
//		OnChangeBorderwidth_edit(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
//
//	Parameters;
//
//	Return Values:
//
//	Comments:
//		Function handles message for change in border width value in edit control.
//
	
LRESULT
CAppearanceProp::OnChangeBorderwidth_edit(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
	SetDirty (TRUE);
	m_pPageSite->OnStatusChange (PROPPAGESTATUS_DIRTY | PROPPAGESTATUS_VALIDATE);

	return 0;
}

//
//	Function:
//		OnDeltaposBorderwidth_spin(int idCtrl, LPNMHDR pnmh, BOOL& bHandled)
//
//	Parameters;
//
//	Return Values:
//
//	Comments:
//		Function handles message for change in spin control for border width of control.
//

LRESULT
CAppearanceProp::OnDeltaposBorderwidth_spin(int idCtrl, LPNMHDR pnmh, BOOL& bHandled)
{
	// TODO : Add Code for control notification handler.
	return 0;
}

//
//	Function:
//		OnClickedEnable_check(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
//
//	Parameters;
//
//	Return Values:
//
//	Comments:
//		Function handles message for change in check box state for control enable state.
//

LRESULT
CAppearanceProp::OnClickedEnable_check(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
	SetDirty (TRUE);
	m_pPageSite->OnStatusChange (PROPPAGESTATUS_DIRTY | PROPPAGESTATUS_VALIDATE);

	return 0;
}

//
//	Function:
//		OnSelchangeFillstyle_combo(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
//
//	Parameters;
//
//	Return Values:
//
//	Comments:
//		Function handles message for chenge in the fill style in the combo box.
//

LRESULT
CAppearanceProp::OnSelchangeFillstyle_combo(WORD wNotifyCode, WORD wID, HWND hWndCtl,
											BOOL& bHandled)
{
	// If the style has been changed to hatched, enable the hatch pattern combo box.

	long style;

	style = SendDlgItemMessage (IDC_FILLSTYLE_COMBO, CB_GETCURSEL, 0, 0);

	::EnableWindow (m_hFillPatternCombo, (style == 2) ? TRUE : FALSE);

	SetDirty (TRUE);

	m_pPageSite->OnStatusChange (PROPPAGESTATUS_DIRTY | PROPPAGESTATUS_VALIDATE);

	return 0;
}

//
//	Function:
//		OnClickedTabstop_check(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
//
//	Parameters;
//
//	Return Values:
//
//	Comments:
//		Function handles message for change in check box state for tab stop state of control.
//

LRESULT
CAppearanceProp::OnClickedTabstop_check(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
	SetDirty (TRUE);
	m_pPageSite->OnStatusChange (PROPPAGESTATUS_DIRTY | PROPPAGESTATUS_VALIDATE);

	return 0;
}

//
//	Function:
//		OnSelchangePattern_combo(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
//
//	Parameters;
//
//	Return Values:
//
//	Comments:
//		Function handles message for change in selection for fill pattern combo box.
//

LRESULT
CAppearanceProp::OnSelchangePattern_combo(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
	SetDirty (TRUE);
	m_pPageSite->OnStatusChange (PROPPAGESTATUS_DIRTY | PROPPAGESTATUS_VALIDATE);

	return 0;
}

//
//	Function:
//		OnSelchangeDrawstyle_combo(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
//
//	Parameters;
//
//	Return Values:
//
//	Comments:
//		Function handles message for change in combo box selection for control draw style.
//

LRESULT
CAppearanceProp::OnSelchangeDrawstyle_combo(WORD wNotifyCode, WORD wID, HWND hWndCtl,
											BOOL& bHandled)
{
	BOOL enableWidthEdit = FALSE;
	long style;

	style = SendDlgItemMessage (IDC_DRAWSTYLE_COMBO, CB_GETCURSEL, 0, 0);

	switch (style) {
		case 0:
			m_nDrawStyle = PS_SOLID;
			break;
		case 1:
			m_nDrawStyle = PS_DASH;
			break;
		case 2:
			m_nDrawStyle = PS_DOT;
			break;
		case 3:
			m_nDrawStyle = PS_DASHDOT;
			break;
		case 4:
			m_nDrawStyle = PS_DASHDOTDOT;
			break;
		case 5:
			m_nDrawStyle = PS_NULL;
			break;
		case 6:
			m_nDrawStyle = PS_INSIDEFRAME;
			break;
	}

	// Enable the width edit control only if the line style is one of Invisible, Solid &
	// Inside frame.

	enableWidthEdit = (m_nDrawStyle == PS_SOLID || m_nDrawStyle == PS_NULL ||
			m_nDrawStyle == PS_INSIDEFRAME) ? TRUE : FALSE;

	::EnableWindow (m_hBorderWidthEdit, enableWidthEdit);
	::EnableWindow (m_hBorderWidthSpin, enableWidthEdit);

	SetDirty (TRUE);
	m_pPageSite->OnStatusChange (PROPPAGESTATUS_DIRTY | PROPPAGESTATUS_VALIDATE);

	return 0;
}

//
//	Function:
//		HandleError ()
//
//	Parameters;
//
//	Return Values:
//
//	Comments:
//		Function handles the display of error messages set in the IErrorInfo interface for control.
//

void
CAppearanceProp::HandleError ()
{
	USES_CONVERSION;

	// Get the IErrorInfo interface

	CComPtr<IErrorInfo> pError;
	CComBSTR strError;

	if (pError != NULL) {
		pError->GetDescription (&strError);
		MessageBox (OLE2T (strError), _T ("Error"), MB_ICONINFORMATION);
	}
}