/////////////////////////////////////////////////////////////////////////////
// ShapeCtl.h : Declaration of the CShapeCtl
//
// Simple ActiveX Control
//
// All rights reserved.
//
// Written by Naveen Kohli (naveen@a3ds.com)
// Version 1.0
//
// Distribute freely, except: don't remove my name from the source or
// documentation (don't take credit for my work), mark your changes (don't
// get me blamed for your possible bugs), don't alter or remove this
// notice.
// No warrantee of any kind, express or implied, is included with this
// software; use at your own risk, responsibility for damages (if any) to
// anyone resulting from the use of this software rests entirely with the
// user.
//
// Send bug reports, bug fixes, enhancements, requests, flames, etc. to
//    naveen@a3ds.com
/////////////////////////////////////////////////////////////////////////////

#ifndef __SHAPECTL_H_
#define __SHAPECTL_H_

#include "resource.h"       // main symbols
#include <atlctl.h>
#include "ActiveXCtlCP.h"

#define SZSTREAM                    OLETEXT("SHAPECTLDATA")

enum {VTEX_FIXED_SIZE_BSTR_BLOB = 999};

using namespace IShapeDispIDs;

/////////////////////////////////////////////////////////////////////////////
// CShapeCtl
class ATL_NO_VTABLE CShapeCtl : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public CStockPropImpl<CShapeCtl, IShapeCtl, &IID_IShapeCtl, &LIBID_ACTIVEXCTLLib>,
	public CComControl<CShapeCtl>,
	public IPersistStreamInitImpl<CShapeCtl>,
	public IOleControlImpl<CShapeCtl>,
	public IOleObjectImpl<CShapeCtl>,
	public IOleInPlaceActiveObjectImpl<CShapeCtl>,
	public IViewObjectExImpl<CShapeCtl>,
	public IOleInPlaceObjectWindowlessImpl<CShapeCtl>,
	public ISupportErrorInfo,
	public IConnectionPointContainerImpl<CShapeCtl>,
	public IPersistStorageImpl<CShapeCtl>,
	public ISpecifyPropertyPagesImpl<CShapeCtl>,
	public IQuickActivateImpl<CShapeCtl>,
	public IDataObjectImpl<CShapeCtl>,
	public IProvideClassInfo2Impl<&CLSID_ShapeCtl, &DIID__IShapeCtlEvents, &LIBID_ACTIVEXCTLLib>,
	public IPropertyNotifySinkCP<CShapeCtl>,
	public CComCoClass<CShapeCtl, &CLSID_ShapeCtl>,
	public CProxy_IShapeCtlEvents< CShapeCtl >,
	public IPersistPropertyBagImpl<CShapeCtl>,
	public IObjectSafetyImpl<CShapeCtl, INTERFACESAFE_FOR_UNTRUSTED_CALLER |
										INTERFACESAFE_FOR_UNTRUSTED_DATA>
{
public:
	CShapeCtl()
	{
		// Set a default size of control at initial creation. Object wizard sets the default
		// size to 5080x5080 himetric units (each unit = 0.01mm).

		SIZE sz = {120, 120};

		// Convert pixels to himetric.
		AtlPixelToHiMetric (&sz, &m_sizeExtent);
		// Store natural extent
		m_sizeNatural = m_sizeExtent;

		m_nRadius = 50;
		m_bstrText = _T ("Naveen's Shape Control");
		m_bstrAuthor = _T ("Naveen K Kohli");
		m_bstrEmail = _T ("naveen@a3ds.com");
		m_bstrDesc = _T ("COM programming using ATL to create ActiveX controls");

		m_nFillStyle = 0;
		m_nFillPattern = fillCrossHatch;
		m_nAppearance = 1;
		m_nDrawWidth = 0;
		m_nDrawStyle = PS_SOLID;
		m_bTabStop = TRUE;
		m_bEnabled = TRUE;
		m_bBorderVisible = TRUE;
		m_clrFillColor = RGB (0, 0, 255);
		m_clrBorderColor = RGB (255, 0, 0);

		// Create the default font object.

		static FONTDESC fontDesc = {sizeof (FONTDESC), OLESTR ("MS Sans Serif"), FONTSIZE (8),
			FW_BOLD, ANSI_CHARSET, FALSE, FALSE, FALSE};

		OleCreateFontIndirect (&fontDesc, IID_IFontDisp, (void **)&m_pFont);

	}

DECLARE_REGISTRY_RESOURCEID(IDR_SHAPECTL)

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(CShapeCtl)
	COM_INTERFACE_ENTRY(IShapeCtl)
	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY(IViewObjectEx)
	COM_INTERFACE_ENTRY(IViewObject2)
	COM_INTERFACE_ENTRY(IViewObject)
	COM_INTERFACE_ENTRY(IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY(IOleInPlaceObject)
	COM_INTERFACE_ENTRY2(IOleWindow, IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY(IOleInPlaceActiveObject)
	COM_INTERFACE_ENTRY(IOleControl)
	COM_INTERFACE_ENTRY(IOleObject)
	COM_INTERFACE_ENTRY(IPersistStreamInit)
	COM_INTERFACE_ENTRY2(IPersist, IPersistStreamInit)
	COM_INTERFACE_ENTRY(ISupportErrorInfo)
	COM_INTERFACE_ENTRY(IConnectionPointContainer)
	COM_INTERFACE_ENTRY(IPersistPropertyBag)
	COM_INTERFACE_ENTRY(ISpecifyPropertyPages)
	COM_INTERFACE_ENTRY(IQuickActivate)
	COM_INTERFACE_ENTRY(IPersistStorage)
	COM_INTERFACE_ENTRY(IDataObject)
	COM_INTERFACE_ENTRY(IProvideClassInfo)
	COM_INTERFACE_ENTRY(IProvideClassInfo2)
	COM_INTERFACE_ENTRY(IObjectSafety)
	COM_INTERFACE_ENTRY_IMPL(IConnectionPointContainer)
END_COM_MAP()

BEGIN_PROP_MAP(CShapeCtl)
	PROP_DATA_ENTRY("_cx", m_sizeExtent.cx, VT_UI4)
	PROP_DATA_ENTRY("_cy", m_sizeExtent.cy, VT_UI4)
	PROP_ENTRY("Appearance", DISPID_APPEARANCE, CLSID_AppearanceProp)
	PROP_ENTRY("AutoSize", DISPID_AUTOSIZE, CLSID_NULL)
	PROP_ENTRY("BackColor", DISPID_BACKCOLOR, CLSID_StockColorPage)
	PROP_ENTRY("BackStyle", DISPID_BACKSTYLE, CLSID_NULL)
	PROP_ENTRY("BorderColor", DISPID_BORDERCOLOR, CLSID_StockColorPage)
	PROP_ENTRY("BorderStyle", DISPID_BORDERSTYLE, CLSID_NULL)
	PROP_ENTRY("BorderVisible", DISPID_BORDERVISIBLE, CLSID_AppearanceProp)
	PROP_ENTRY("BorderWidth", DISPID_BORDERWIDTH, CLSID_NULL)
	PROP_ENTRY("Caption", DISPID_CAPTION, CLSID_NULL)
	PROP_ENTRY("DrawMode", DISPID_DRAWMODE, CLSID_NULL)
	PROP_ENTRY("DrawStyle", DISPID_DRAWSTYLE, CLSID_AppearanceProp)
	PROP_ENTRY("DrawWidth", DISPID_DRAWWIDTH, CLSID_AppearanceProp)
	PROP_ENTRY("Enabled", DISPID_ENABLED, CLSID_AppearanceProp)
	PROP_ENTRY("FillColor", DISPID_FILLCOLOR, CLSID_StockColorPage)
	PROP_ENTRY("FillStyle", DISPID_FILLSTYLE, CLSID_AppearanceProp)
	PROP_ENTRY("Font", DISPID_FONT, CLSID_StockFontPage)
	PROP_ENTRY("ForeColor", DISPID_FORECOLOR, CLSID_StockColorPage)
	PROP_ENTRY("HWND", DISPID_HWND, CLSID_NULL)
	PROP_ENTRY("MouseIcon", DISPID_MOUSEICON, CLSID_StockPicturePage)
	PROP_ENTRY("MousePointer", DISPID_MOUSEPOINTER, CLSID_NULL)
	PROP_ENTRY("Picture", DISPID_PICTURE, CLSID_StockPicturePage)
	PROP_ENTRY("TabStop", DISPID_TABSTOP, CLSID_AppearanceProp)
	PROP_ENTRY("Text", DISPID_TEXT, CLSID_ShapeProp)
	PROP_ENTRY("Valid", DISPID_VALID, CLSID_NULL)
	PROP_ENTRY("Radius", 1, CLSID_ShapeProp)
	PROP_ENTRY("FillPattern", dispid_FillPattern, CLSID_AppearanceProp)
	// Example entries
	// PROP_ENTRY("Property Description", dispid, clsid)
	// PROP_PAGE(CLSID_StockColorPage)
END_PROP_MAP()

BEGIN_CONNECTION_POINT_MAP(CShapeCtl)
	CONNECTION_POINT_ENTRY(IID_IPropertyNotifySink)
	CONNECTION_POINT_ENTRY(DIID__IShapeCtlEvents)
END_CONNECTION_POINT_MAP()

BEGIN_MSG_MAP(CShapeCtl)
	CHAIN_MSG_MAP(CComControl<CShapeCtl>)
	DEFAULT_REFLECTION_HANDLER()
	MESSAGE_HANDLER(WM_LBUTTONDBLCLK, OnLButtonDblClk)
	MESSAGE_HANDLER(WM_LBUTTONDOWN, OnLButtonDown)
	MESSAGE_HANDLER(WM_MOUSEMOVE, OnMouseMove)
	MESSAGE_HANDLER(WM_CREATE, OnCreate)
END_MSG_MAP()
// Handler prototypes:
//  LRESULT MessageHandler(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
//  LRESULT CommandHandler(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
//  LRESULT NotifyHandler(int idCtrl, LPNMHDR pnmh, BOOL& bHandled);

// IObjectSafety
	STDMETHOD(GetInterfaceSafetyOptions)(REFIID riid, DWORD *pdwSupportedOptions,
		DWORD *pdwEnabledOptions);

	STDMETHOD(SetInterfaceSafetyOptions)(REFIID riid, DWORD dwOptionSetMask,
		DWORD dwEnabledOptions);

// IOleControl
	STDMETHOD(OnAmbientPropertyChange)(DISPID dispid);

// ISupportsErrorInfo
	STDMETHOD(InterfaceSupportsErrorInfo)(REFIID riid);

// IPersistStreamImpl
	STDMETHOD(Load)(IStream *pStream);
	STDMETHOD(Save)(IStream *pStream, BOOL fClearDirty);
	STDMETHOD(GetSizeMax)(ULARGE_INTEGER* pcbSize);

// IPersistStorageImpl
	STDMETHOD(Load)(IStorage *Storage); 
	STDMETHOD(Save)(IStorage *pStorage, BOOL fSameAsLoad);

// IPersistPropertBag
	STDMETHOD(Load)(IPropertyBag *pPropBag, IErrorLog* pErrorLog);
	STDMETHOD(Save)(IPropertyBag *pPropBag, BOOL fClearDirty, BOOL fSaveAllProperties);

// IViewObjectEx
	DECLARE_VIEW_STATUS(VIEWSTATUS_SOLIDBKGND | VIEWSTATUS_OPAQUE)

// IShapeCtl
public:
	STDMETHOD(get_FillPattern)(/*[out, retval]*/ FILLPATTERN *pVal);
	STDMETHOD(put_FillPattern)(/*[in]*/ FILLPATTERN newVal);
	STDMETHOD(get_Radius)(/*[out, retval]*/ short *pVal);
	STDMETHOD(put_Radius)(/*[in]*/ short newVal);

	HRESULT OnDraw(ATL_DRAWINFO& di);
	LRESULT OnLButtonDblClk(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
	LRESULT OnLButtonDown(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
	LRESULT OnMouseMove(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
	LRESULT OnCreate(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
	
	short m_nAppearance;
	OLE_COLOR m_clrBackColor;
	LONG m_nBackStyle;
	OLE_COLOR m_clrBorderColor;
	LONG m_nBorderStyle;
	BOOL m_bBorderVisible;
	LONG m_nBorderWidth;
	CComBSTR m_bstrCaption;
	LONG m_nDrawMode;
	LONG m_nDrawStyle;
	LONG m_nDrawWidth;
	BOOL m_bEnabled;
	OLE_COLOR m_clrFillColor;
	LONG m_nFillStyle;
	CComPtr<IFontDisp> m_pFont;
	OLE_COLOR m_clrForeColor;
	CComPtr<IPictureDisp> m_pMouseIcon;
	LONG m_nMousePointer;
	CComPtr<IPictureDisp> m_pPicture;
	BOOL m_bTabStop;
	CComBSTR m_bstrText;
	BOOL m_bValid;
	short m_nRadius;
	FILLPATTERN m_nFillPattern;
	CComBSTR m_bstrAuthor;
	CComBSTR m_bstrEmail;
	CComBSTR m_bstrDesc;

private:
	IStorage *m_pIStorage;
	void Insert (BSTR *stream, void *pData, DWORD size);
	void Extract (LPCWSTR *stream, void *pData, DWORD size);
};

#endif //__SHAPECTL_H_
