/////////////////////////////////////////////////////////////////////////////
// ShapeProp.cpp : Implementation of CShapeProp
//
// Property page for control's properties
//
// All rights reserved.
//
// Written by Naveen Kohli (naveen@a3ds.com)
// Version 1.0
//
// Distribute freely, except: don't remove my name from the source or
// documentation (don't take credit for my work), mark your changes (don't
// get me blamed for your possible bugs), don't alter or remove this
// notice.
// No warrantee of any kind, express or implied, is included with this
// software; use at your own risk, responsibility for damages (if any) to
// anyone resulting from the use of this software rests entirely with the
// user.
//
// Send bug reports, bug fixes, enhancements, requests, flames, etc. to
//    naveen@a3ds.com
/////////////////////////////////////////////////////////////////////////////
 
#include "stdafx.h"
#include "ActiveXCtl.h"
#include "ShapeProp.h"

/////////////////////////////////////////////////////////////////////////////
// CShapeProp

//
//	Function:
//		Apply (void)
//
//	Parameters;
//
//	Return Values:
//		S_OK	Control values applied successfully
//		E_FAIL	Error in applying the control values
//
//	Comments:
//		Function updates the values of control from the values in the property page.
//

STDMETHODIMP
CShapeProp::Apply (void)
{
	USES_CONVERSION;

	// NOTE: There may be different applications referencing to this control. Therefor we have to
	//		 process the loop and get IUnkown pointer for each site and do the processing.

	for (UINT i = 0; i < m_nObjects; i++)
	{
		LPOLESTR lpStr = NULL;
		CComQIPtr<IShapeCtl, &IID_IShapeCtl> pShapeCtl (m_ppUnk[i]);
		
		// Get the text from edit control.

		if (GetDlgItemText (IDC_TEXT_EDIT, lpStr)) {
			if (FAILED (pShapeCtl->put_Text (lpStr))) {
				HandleError ();
				return (E_FAIL);
			}
		}

		// Set the radius

		short radius;

		radius = GetDlgItemInt (IDC_RADIUS_EDIT, NULL, NULL);
		if (FAILED (pShapeCtl->put_Radius (radius))) {
			HandleError ();
			return (E_FAIL);
		}
	}

	m_bDirty = FALSE;
	return S_OK;
}

//
//	Function:
//		OnInitDialog(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
//
//	Parameters;
//
//	Return Values:
//		S_OK	Successful property page intialization
//		E_FAIL	Error in property page intialization
//
//	Comments:
//		Function handles the property page intialization.
//

LRESULT
CShapeProp::OnInitDialog(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	USES_CONVERSION;

	// Lets set some ranges for controls.

	if ((m_hRadiusEdit = GetDlgItem (IDC_RADIUS_EDIT)) != NULL) {
		::SendMessage (m_hRadiusEdit, UDM_SETRANGE, 0, MAKELONG (100, 0));
	}

	if ((m_hRadiusSpin = GetDlgItem (IDC_RADIUS_SPIN)) != NULL) {
		::SendMessage (m_hRadiusSpin, UDM_SETRANGE, 0, MAKELONG (100, 0));
	}

	// Initialize all the controls.

	if (m_nObjects > 0) {
		CComQIPtr<IShapeCtl, &IID_IShapeCtl> pShapeCtl (m_ppUnk[0]);
		BSTR text;
		short radius;

		// Get the text from the control and set in the edit control.

		if (SUCCEEDED (pShapeCtl->get_Text (&text))) {
			SetDlgItemText (IDC_TEXT_EDIT, OLE2T (text));
		}

		// Set the radius.

		if (SUCCEEDED (pShapeCtl->get_Radius (&radius))) {
			SetDlgItemInt (IDC_RADIUS_EDIT, radius, FALSE);
			::SendMessage (m_hRadiusSpin, UDM_SETPOS, 0, MAKELONG (radius, 0));
		}
	}

	return 0;
}

//
//	Function:
//		OnChangeRadius_edit(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
//
//	Parameters;
//
//	Return Values:
//
//	Comments:
//		Function handles change in radius values in edit control. 
//

LRESULT
CShapeProp::OnChangeRadius_edit(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
	SetDirty (TRUE);
	m_pPageSite->OnStatusChange (PROPPAGESTATUS_DIRTY | PROPPAGESTATUS_VALIDATE);

	return 0;
}

//
//	Function:
//		OnDeltaposRadius_spin(int idCtrl, LPNMHDR pnmh, BOOL& bHandled)
//
//	Parameters;
//
//	Return Values:
//
//	Comments:
//		Function handles the change in spin control for radius values.
//

LRESULT
CShapeProp::OnDeltaposRadius_spin(int idCtrl, LPNMHDR pnmh, BOOL& bHandled)
{
	// TODO : Add Code for control notification handler.
	return 0;
}

//
//	Function:
//		OnChangeText_edit(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
//
//	Parameters;
//
//	Return Values:
//
//	Comments:
//		Function handles the change in text to be displayed in control.

LRESULT
CShapeProp::OnChangeText_edit(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
	SetDirty (TRUE);
	m_pPageSite->OnStatusChange (PROPPAGESTATUS_DIRTY | PROPPAGESTATUS_VALIDATE);
	return 0;
}

//
//	Function:
//		HandleError ()
//
//	Parameters;
//
//	Return Values:
//
//	Comments:
//		Function handles the display of error messages set in the IErrorInfo interface for control.
//

void
CShapeProp::HandleError ()
{
	USES_CONVERSION;

	// Get the IErrorInfo interface

	CComPtr<IErrorInfo> pError;
	CComBSTR strError;

	if (pError != NULL) {
		pError->GetDescription (&strError);
		MessageBox (OLE2T (strError), _T ("Error"), MB_ICONINFORMATION);
	}
}
