/////////////////////////////////////////////////////////////////////////////
// ShapeProp.h : Declaration of the CShapeProp
//
// Property page for control properties.
//
// All rights reserved.
//
// Written by Naveen Kohli (naveen@a3ds.com)
// Version 1.0
//
// Distribute freely, except: don't remove my name from the source or
// documentation (don't take credit for my work), mark your changes (don't
// get me blamed for your possible bugs), don't alter or remove this
// notice.
// No warrantee of any kind, express or implied, is included with this
// software; use at your own risk, responsibility for damages (if any) to
// anyone resulting from the use of this software rests entirely with the
// user.
//
// Send bug reports, bug fixes, enhancements, requests, flames, etc. to
//    naveen@a3ds.com
/////////////////////////////////////////////////////////////////////////////

#ifndef __SHAPEPROP_H_
#define __SHAPEPROP_H_

#include "resource.h"       // main symbols
#include <commctrl.h>

EXTERN_C const CLSID CLSID_ShapeProp;

/////////////////////////////////////////////////////////////////////////////
// CShapeProp
class ATL_NO_VTABLE CShapeProp :
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<CShapeProp, &CLSID_ShapeProp>,
	public IPropertyPageImpl<CShapeProp>,
	public CDialogImpl<CShapeProp>
{
public:
	CShapeProp() 
	{
		m_dwTitleID = IDS_TITLEShapeProp;
		m_dwHelpFileID = IDS_HELPFILEShapeProp;
		m_dwDocStringID = IDS_DOCSTRINGShapeProp;
	}

	enum {IDD = IDD_SHAPEPROP};
	HWND m_hRadiusEdit;
	HWND m_hRadiusSpin;

DECLARE_REGISTRY_RESOURCEID(IDR_SHAPEPROP)

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(CShapeProp) 
	COM_INTERFACE_ENTRY(IPropertyPage)
END_COM_MAP()

BEGIN_MSG_MAP(CShapeProp)
	CHAIN_MSG_MAP(IPropertyPageImpl<CShapeProp>)
	COMMAND_HANDLER(IDC_RADIUS_EDIT, EN_CHANGE, OnChangeRadius_edit)
	NOTIFY_HANDLER(IDC_RADIUS_SPIN, UDN_DELTAPOS, OnDeltaposRadius_spin)
	COMMAND_HANDLER(IDC_TEXT_EDIT, EN_CHANGE, OnChangeText_edit)
	MESSAGE_HANDLER(WM_INITDIALOG, OnInitDialog)
END_MSG_MAP()
// Handler prototypes:
//  LRESULT MessageHandler(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
//  LRESULT CommandHandler(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
//  LRESULT NotifyHandler(int idCtrl, LPNMHDR pnmh, BOOL& bHandled);

	STDMETHOD(Apply)(void);
	LRESULT OnChangeRadius_edit(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
	LRESULT OnDeltaposRadius_spin(int idCtrl, LPNMHDR pnmh, BOOL& bHandled);
	LRESULT OnChangeText_edit(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
	LRESULT OnInitDialog(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);

private:
	void HandleError ();
};

#endif //__SHAPEPROP_H_
